==> unicore-ucc6-5.0.0-1.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/actions/ACL.java <==
/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 27-07-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package de.fzj.unicore.ucc.actions;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.OptionBuilder;
import org.unigrids.services.atomic.types.ACLEntryTypeType;
import org.unigrids.x2006.x04.services.sms.ACLChangeModeType;
import org.unigrids.x2006.x04.services.sms.ChangeACLEntryType;

import de.fzj.unicore.uas.client.StorageClient;

/**
 * Changes ACL of a given path.
 * 
 * @author K. Benedyczak
 */
public class ACL extends SMSOperation {
	public static final String OPT_CLEAN_LONG = "clean";
	public static final String OPT_CLEAN = "b";
	public static final String OPT_DELETE_LONG = "delete";
	public static final String OPT_DELETE = "d";
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_RECURSIVE_LONG)
				.withDescription("Recursively applies the operation to the whole contents of the directory specified as an argument.")
				.isRequired(false)
				.create(OPT_RECURSIVE)
				);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_CLEAN_LONG)
				.withDescription("Prior to applying all other ACLs (if any are present) all ACL of the path are cleaned.")
				.isRequired(false)
				.create(OPT_CLEAN)
				);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_DELETE_LONG)
				.withDescription("Specified ACEs are deleted from the path's ACL (if this option is not specified then ACEs are added).")
				.isRequired(false)
				.create(OPT_DELETE)
				);
	}

	@Override
	public String getName() {
		return "setacl";
	}

	@Override
	public String getSynopsis() {
		return "Changes ACL of a file on a remote storage. By default the command will add " +
				"the specified Access Control Entries (ACEs) to the path's ACL. With the '"
				+ OPT_DELETE_LONG + "' option the ACEs are removed. The ACEs specification is " +
				"required always unless the '" + OPT_CLEAN_LONG + "' option is specified. " +
				"The ACE syntax is as follows:\n\n" +
				"[d:]<g|u>:[subject]:[permissons]\n\n" +
				"where: 'd' stands for the default directory ACL, " +
				"'g' or 'u' for the group or user ACL. Permissions are in the 'rwx' format " +
				"with '-' where appropriate. " +
				"Note that permissions part is required always unless the ACE is going to be removed.\n" +
				"Example specification of two ACEs suitable for adding to a directory:\n" +
				"g:wheel:rwx,d:u::r-x\n" +
				"This example will add rwx permissions to the wheel group and set the default user ACL r-x " +
				"(the default ACL will be used for the contents subsequently created in the directory).";
	}
	
	@Override
	public String getDescription(){
		return "change file's ACL";
	}
	
	@Override
	public String getArgumentList(){
		return "[ACE[,ACE...]] <[Storage-URL#]/path or [u6://SITENAME/[JobId|StorageName]]/path>";
	}
	
	@Override
	public String getCommandGroup(){
		return "Data management";
	}
	
	
	@Override
	public void process() {
		super.process();
		boolean recursive = getBooleanOption(OPT_RECURSIVE_LONG, OPT_RECURSIVE);
		verbose("Recursive invocation = " + recursive);
		boolean clean = getBooleanOption(OPT_CLEAN_LONG, OPT_CLEAN);
		verbose("Clean up ACL = " + clean);
		boolean remove = getBooleanOption(OPT_DELETE_LONG, OPT_DELETE);
		verbose("Delete ACEs = " + remove);
		
		
		CommandLine cmdLine = getCommandLine(); 
		if ((clean && cmdLine.getArgs().length < 2) || (!clean && cmdLine.getArgs().length < 3)) {
			error("Not enough arguments", null);
			printUsage();
			endProcessing(ERROR_CLIENT);
		}
		
		ChangeACLEntryType[] spec = null;
		String target;
		if (cmdLine.getArgs().length > 2) {
			spec = parseACEs(cmdLine.getArgs()[1], remove);
			target = cmdLine.getArgs()[2];
		} else
			target = cmdLine.getArgs()[1];
		
		
		verbose("Modifying ACLs of: " + target);
		
		StorageClient sms = getStorageClient(target);
		
		try{
			String path = getPathAtStorage(target);
			
			sms.setACL(path, clean, spec, recursive);
		} catch (Exception ex) {
			error("Error during ACL manipulation.", ex);
			endProcessing(ERROR);
		}
	}
	
	
	private ChangeACLEntryType[] parseACEs(String spec, boolean remove) {
		String[] specA = spec.split(",");
		ChangeACLEntryType ret[] = new ChangeACLEntryType[specA.length];
		for (int i=0; i<specA.length; i++) {
			ret[i] = ChangeACLEntryType.Factory.newInstance();
			if (!remove)
				ret[i].setMode(ACLChangeModeType.MODIFY);
			else
				ret[i].setMode(ACLChangeModeType.REMOVE);
			
			
			String []entries = splitStrict(specA[i]);
			if (entries.length < 3)
				errorACE(specA[i]);
			
			int off = 0;
			ret[i].setDefaultACL(false);
			if (entries[0].equals("d")) {
				off++;
				ret[i].setDefaultACL(true);
				if (entries.length < 4)
					errorACE(specA[i]);
			}
			
			if (entries[off].equals("u")) {
				ret[i].setKind(ACLEntryTypeType.USER);
			} else if (entries[off].equals("g")) {
				ret[i].setKind(ACLEntryTypeType.GROUP);				
			} else {
				errorACE(specA[i]);
			}
			
			ret[i].setSubject(entries[++off]);
			
			if (remove)
				ret[i].setPermissions("");
			else {
				if (entries.length == ++off)
					errorACE(specA[i]);
				if (!Chmod.pattern.matcher(entries[off]).matches())
					errorACE(specA[i]);
				ret[i].setPermissions(entries[off]);
			}
			if (isVerbose())
				verbose("Parsed " + specA[i] + " into:\n" + ret[i].xmlText());
		}
		return ret;
	}
	
	private void errorACE(String ace) {
		error("Invalid format of ACE: " + ace, null);
		printUsage();
		endProcessing(ERROR_CLIENT);		
	}
	
	private String[] splitStrict(String toSplit) {
		List<String> retList = new ArrayList<String>();
		char[] chars = toSplit.toCharArray();
		StringBuilder sb = new StringBuilder();
		for (int i=0; i<chars.length; i++) {
			if (chars[i] == ':') {
				retList.add(sb.toString());
				sb = new StringBuilder();
			} else {
				sb.append(chars[i]);
			}
		}
		retList.add(sb.toString());
		return retList.toArray(new String[retList.size()]);
	}
}

==> unicore-ucc6-6.0.0-0.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/actions/ACL.java <==
/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 27-07-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package de.fzj.unicore.ucc.actions;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.OptionBuilder;
import org.unigrids.services.atomic.types.ACLEntryTypeType;
import org.unigrids.x2006.x04.services.sms.ACLChangeModeType;
import org.unigrids.x2006.x04.services.sms.ChangeACLEntryType;

import de.fzj.unicore.uas.client.StorageClient;

/**
 * Changes ACL of a given path.
 * 
 * @author K. Benedyczak
 */
public class ACL extends SMSOperation {
	public static final String OPT_CLEAN_LONG = "clean";
	public static final String OPT_CLEAN = "b";
	public static final String OPT_DELETE_LONG = "delete";
	public static final String OPT_DELETE = "d";
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_RECURSIVE_LONG)
				.withDescription("Recursively applies the operation to the whole contents of the directory specified as an argument.")
				.isRequired(false)
				.create(OPT_RECURSIVE)
				);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_CLEAN_LONG)
				.withDescription("Prior to applying all other ACLs (if any are present) all ACL of the path are cleaned.")
				.isRequired(false)
				.create(OPT_CLEAN)
				);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_DELETE_LONG)
				.withDescription("Specified ACEs are deleted from the path's ACL (if this option is not specified then ACEs are added).")
				.isRequired(false)
				.create(OPT_DELETE)
				);
	}

	@Override
	public String getName() {
		return "setacl";
	}

	@Override
	public String getSynopsis() {
		return "Changes ACL of a file on a remote storage. By default the command will add " +
				"the specified Access Control Entries (ACEs) to the path's ACL. With the '"
				+ OPT_DELETE_LONG + "' option the ACEs are removed. The ACEs specification is " +
				"required always unless the '" + OPT_CLEAN_LONG + "' option is specified. " +
				"The ACE syntax is as follows:\n\n" +
				"[d:]<g|u>:[subject]:[permissons]\n\n" +
				"where: 'd' stands for the default directory ACL, " +
				"'g' or 'u' for the group or user ACL. Permissions are in the 'rwx' format " +
				"with '-' where appropriate. Capital X is also supported." +
				"Note that permissions part is required always unless the ACE is going to be removed.\n" +
				"Example specification of two ACEs suitable for adding to a directory:\n" +
				"g:wheel:rwx,d:u::r-x\n" +
				"This example will add rwx permissions to the wheel group and set the default user ACL r-x " +
				"(the default ACL will be used for the contents subsequently created in the directory).";
	}
	
	@Override
	public String getDescription(){
		return "change file's ACL";
	}
	
	@Override
	public String getArgumentList(){
		return "[ACE[,ACE...]] <[Storage-URL#]/path or [u6://SITENAME/[JobId|StorageName]]/path>";
	}
	
	@Override
	public String getCommandGroup(){
		return "Data management";
	}
	
	
	@Override
	public void process() {
		super.process();
		boolean recursive = getBooleanOption(OPT_RECURSIVE_LONG, OPT_RECURSIVE);
		verbose("Recursive invocation = " + recursive);
		boolean clean = getBooleanOption(OPT_CLEAN_LONG, OPT_CLEAN);
		verbose("Clean up ACL = " + clean);
		boolean remove = getBooleanOption(OPT_DELETE_LONG, OPT_DELETE);
		verbose("Delete ACEs = " + remove);
		
		
		CommandLine cmdLine = getCommandLine(); 
		if ((clean && cmdLine.getArgs().length < 2) || (!clean && cmdLine.getArgs().length < 3)) {
			error("Not enough arguments", null);
			printUsage();
			endProcessing(ERROR_CLIENT);
		}
		
		ChangeACLEntryType[] spec = null;
		String target;
		if (cmdLine.getArgs().length > 2) {
			spec = parseACEs(cmdLine.getArgs()[1], remove);
			target = cmdLine.getArgs()[2];
		} else
			target = cmdLine.getArgs()[1];
		
		
		verbose("Modifying ACLs of: " + target);
		
		StorageClient sms = getStorageClient(target);
		
		try{
			String path = getPathAtStorage(target);
			
			sms.setACL(path, clean, spec, recursive);
		} catch (Exception ex) {
			error("Error during ACL manipulation.", ex);
			endProcessing(ERROR);
		}
	}
	
	
	private ChangeACLEntryType[] parseACEs(String spec, boolean remove) {
		String[] specA = spec.split(",");
		ChangeACLEntryType ret[] = new ChangeACLEntryType[specA.length];
		for (int i=0; i<specA.length; i++) {
			ret[i] = ChangeACLEntryType.Factory.newInstance();
			if (!remove)
				ret[i].setMode(ACLChangeModeType.MODIFY);
			else
				ret[i].setMode(ACLChangeModeType.REMOVE);
			
			
			String []entries = splitStrict(specA[i]);
			if (entries.length < 3)
				errorACE(specA[i]);
			
			int off = 0;
			ret[i].setDefaultACL(false);
			if (entries[0].equals("d")) {
				off++;
				ret[i].setDefaultACL(true);
				if (entries.length < 4)
					errorACE(specA[i]);
			}
			
			if (entries[off].equals("u")) {
				ret[i].setKind(ACLEntryTypeType.USER);
			} else if (entries[off].equals("g")) {
				ret[i].setKind(ACLEntryTypeType.GROUP);				
			} else {
				errorACE(specA[i]);
			}
			
			ret[i].setSubject(entries[++off]);
			
			if (remove)
				ret[i].setPermissions("");
			else {
				if (entries.length == ++off)
					errorACE(specA[i]);
				if (!Chmod.pattern.matcher(entries[off]).matches())
					errorACE(specA[i]);
				ret[i].setPermissions(entries[off]);
			}
			if (isVerbose())
				verbose("Parsed " + specA[i] + " into:\n" + ret[i].xmlText());
		}
		return ret;
	}
	
	private void errorACE(String ace) {
		error("Invalid format of ACE: " + ace, null);
		printUsage();
		endProcessing(ERROR_CLIENT);		
	}
	
	private String[] splitStrict(String toSplit) {
		List<String> retList = new ArrayList<String>();
		char[] chars = toSplit.toCharArray();
		StringBuilder sb = new StringBuilder();
		for (int i=0; i<chars.length; i++) {
			if (chars[i] == ':') {
				retList.add(sb.toString());
				sb = new StringBuilder();
			} else {
				sb.append(chars[i]);
			}
		}
		retList.add(sb.toString());
		return retList.toArray(new String[retList.size()]);
	}
}
